<?php

namespace Crt\Fussballdaten;

/**
 * Plugin Name:       Fussballdaten
 * Plugin URI:        https://wordpress.org/plugins/fussballdaten/
 * Description:       Get and display data from fussballdaten api. Heavily inspired by Rene Volbach.
 * Version:           1.1.0
 * Author:            Carsten Tetzlaff
 * Author URI:        https://www.fussballdaten.de
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       fussballdaten-data
 * Domain Path:       /wordpress
 */

use Crt\Fussballdaten\src\api\CompetitionRestRoute;
use Crt\Fussballdaten\src\api\MatchdayRestRoute;
use Crt\Fussballdaten\src\api\TableRestRoute;
use Crt\Fussballdaten\src\Cronjob;
use Crt\Fussballdaten\src\Customizer;
use Crt\Fussballdaten\src\page\SettingPage;
use Crt\Fussballdaten\src\shortcode\GeneralShortcode;
use Crt\Fussballdaten\src\Singleton;

if (file_exists($composerAutoload = __DIR__ . '/vendor/autoload.php') /* check in self */
	|| file_exists($composerAutoload = WP_CONTENT_DIR . '/vendor/autoload.php') /* check in wp-content */
	|| file_exists($composerAutoload = plugin_dir_path(__FILE__) . 'vendor/autoload.php') /* check in plugin directory */
	|| file_exists($composerAutoload = get_stylesheet_directory() . '/vendor/autoload.php') /* check in child theme */
	|| file_exists($composerAutoload = get_template_directory() . '/vendor/autoload.php') /* check in parent theme */
) {
	require_once $composerAutoload;
}

class Fussballdaten extends Singleton {

	const API_URI = 'http://api.fussballdaten.de/wp/';

	/** @var mixed|void */
	private $options;

	/** @var mixed|void */
	private $cache;

	public $isValidApiKey = false;
	public $id;

	protected $customizer;

	public function init() {

		// Hooks
		add_action('wp_head', array($this, 'echo_admin_ajax_url'));

		register_activation_hook(__FILE__, array($this, 'add_cron'));
		register_deactivation_hook(__FILE__, array($this, 'remove_cron'));

		add_action('wp_ajax_nopriv_fussballdaten_get_table', array($this, 'ajax_get_table'));
		add_action('wp_ajax_fussballdaten_get_table', array($this, 'ajax_get_table'));

		add_action('wp_ajax_nopriv_fussballdaten_get_matchday', array($this, 'ajax_get_matchday'));
		add_action('wp_ajax_fussballdaten_get_matchday', array($this, 'ajax_get_matchday'));

		Cronjob::get_instance()->init();
		GeneralShortcode::get_instance()->init();
		SettingPage::get_instance()->init();
		Customizer::get_instance()->init();

		$this->checkApiKey();

		add_action('rest_api_init', function () {
			$fussballdatenTableRestRoute       = new TableRestRoute();
			$fussballdatenCompetitionRestRoute = new CompetitionRestRoute();
			$fussballdatenMatchdayRestRoute    = new MatchdayRestRoute();
		});

	}

	// Funktion zur Überprüfung der Gültigkeit des API-Keys
	public function checkApiKey() {
		$api_key = get_option('apiToken');
		// Füge hier deine Logik zur Überprüfung der Gültigkeit des API-Keys hinzu
		// Beispiel: Überprüfe, ob der API-Key gültig ist und setze die globale Variable entsprechend
		$this->isValidApiKey = true;
	}

	/**
	 * @return array
	 */
	public function getOptions() {

		return get_option('fussballdaten_settings', array(
			'apiToken'      => '',
			'competitionId' => '',
		));
	}

	/**
	 * @return mixed|void
	 */
	public function setOptions($options) {
		update_option('fussballdaten_settings', $options);
	}

	/**
	 * @return array
	 */
	public function getCache($competitionId = 0) {

		$caches = [
			'table'       => [
				$competitionId =>
				[
					'response'   => '',
					'lastUpdate' => '',
				],
			],
			'competition' => [
				$competitionId =>
				[
					'response'   => '',
					'lastUpdate' => '',
				],
			],
			'matchdays'   => [
				$competitionId =>
				[
					'response'   => '',
					'lastUpdate' => '',
				],
			],
		];

		return get_option('fussballdaten_cache', $caches);
	}

	/**
	 * @return mixed|void
	 */
	public function setCache($cache) {
		update_option('fussballdaten_cache', $cache);
	}

	/**
	 *  Echos URL of admin-ajax.php into the javascript
	 */
	public function echo_admin_ajax_url() {
		$ajax_url = '"' . admin_url() . 'admin-ajax.php"';
		echo ('<script type="text/javascript"> var ajax_url = ' . $ajax_url . '; </script>');

	}

	/**
	 * add cron
	 */
	public function add_cron() {
		Cronjob::get_instance()->add_cron();
	}

	/**
	 * remove cron
	 */
	public function remove_cron() {
		Cronjob::get_instance()->remove_cron();
	}

	/**
	 * ajax_get_table
	 */
	public function ajax_get_table() {
		header('Content-type: application/json');
		$competitionId = $_GET['id'];
		$return        = array_merge(array('success' => true), $this->getCache()['table'][$competitionId]);

		echo json_encode($return);

		die();
	}

	/**
	 * ajax_get_matchday
	 */
	public function ajax_get_matchday($competitionId) {
		header('Content-type: application/json');

		$cache           = $this->getCache();
		$competitionId   = $_GET['id'];
		$currentMatchday = $cache['competition'][$competitionId]['currentMatchday'];

		$matchday = $cache['matchdays'][$competitionId]['fixtures'][$currentMatchday];

		$return = array('success' => true, 'fixture' => $matchday);

		echo json_encode($return);

		die();
	}
}

Fussballdaten::get_instance()->init();