<?php

namespace Crt\Fussballdaten\src;

use Crt\Fussballdaten\Fussballdaten;
use WP_Customize_Color_Control;

/**
 * Class Customizer
 * @package Crt\Fussballdaten\src;
 */

class Customizer extends Singleton {

	/**
	 * Init
	 */
	public function init() {
		add_action('customize_register', [$this, 'register_customizer_settings']);
	}

	/**
	 * Register Customizer settings
	 */
	public function register_customizer_settings($wp_customize) {

		$wp_customize->add_panel(
			'fdwp_panel',
			[
				'title'       => 'Fussballdaten WP',
				'description' => '', // Include html tags such as <p>.
				'priority'    => 200, // Mixed with top-level-section hierarchy.
			]
		);

		$wp_customize->add_section(
			'fdwp_font_sizes',
			// Arguments array
			[
				'title' => 'Schriftgrößen',
				'panel' => 'fdwp_panel',
			]
		);

		/*
			        |--------------------------------------------------------------------
			        | anwp-text-xxs
			        |--------------------------------------------------------------------
		*/
		$wp_customize->add_setting(
			'fdwp-customizer[font_sizes][table_header_row]',
			[
				'default' => 'default',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			'fdwp-customizer[font_sizes][table_header_row]',
			[
				'type'               => 'number',
				'description_hidden' => false,
				'description'        => 'Default: inherit',
				'label'              => 'Font Size - Table Header',
				'section'            => 'fdwp_font_sizes',
				'settings'           => 'fdwp-customizer[font_sizes][table_header_row]',
			]
		);

		$wp_customize->add_setting(
			'fdwp-customizer[font_sizes][table_row]',
			[
				'default' => 'default',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			'fdwp-customizer[font_sizes][table_row]',
			[
				'type'               => 'number',
				'description_hidden' => false,
				'description'        => 'Default: inherit',
				'label'              => 'Font Size - Table Row',
				'section'            => 'fdwp_font_sizes',
				'settings'           => 'fdwp-customizer[font_sizes][table_row]',
			]
		);

		//=================================
		//-- COLORS --
		//=================================
		$wp_customize->add_section(
			'fdwp_colors',
			[
				'title' => 'Farben',
				'panel' => 'fdwp_panel',
			]
		);

		/*
			        |--------------------------------------------------------------------
			        | fdwp-th
			        |--------------------------------------------------------------------
		*/
		$wp_customize->add_setting(
			'fdwp-customizer[colors][th]',
			[
				'default' => '#ffffff',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[colors][th]',
				[
					'description_hidden' => false,
					'description'        => 'Default: #ffffff',
					'label'              => 'Header Row',
					'section'            => 'fdwp_colors',
					'settings'           => 'fdwp-customizer[colors][th]',
				]
			)
		);

		$wp_customize->add_setting(
			'fdwp-customizer[colors][td]',
			[
				'default' => '#333333',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[colors][td]',
				[
					'description_hidden' => false,
					'description'        => 'Default: #333333',
					'label'              => 'Table Rows',
					'section'            => 'fdwp_colors',
					'settings'           => 'fdwp-customizer[colors][td]',
				]
			)
		);

		$wp_customize->add_setting(
			'fdwp-customizer[colors][bg_th]',
			[
				'default' => '#34ac5d',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[colors][bg_th]',
				[
					'description_hidden' => false,
					'description'        => 'Default: #34ac5d',
					'label'              => 'Background Table Head',
					'section'            => 'fdwp_colors',
					'settings'           => 'fdwp-customizer[colors][bg_th]',
				]
			)
		);

		$wp_customize->add_setting(
			'fdwp-customizer[colors][bg_td]',
			[
				'default' => '',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[colors][bg_td]',
				[
					'description_hidden' => false,
					'description'        => 'Default: ',
					'label'              => 'Background Table Row',
					'section'            => 'fdwp_colors',
					'settings'           => 'fdwp-customizer[colors][bg_td]',
				]
			)
		);

		$wp_customize->add_setting(
			'fdwp-customizer[colors][bg_td_stripe]',
			[
				'default' => '#e9e9e9',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[colors][bg_td_stripe]',
				[
					'description_hidden' => false,
					'description'        => 'Default: #e9e9e9',
					'label'              => 'Background Table Row Stripe',
					'section'            => 'fdwp_colors',
					'settings'           => 'fdwp-customizer[colors][bg_td_stripe]',
				]
			)
		);

/*
|--------------------------------------------------------------------
| fdwp-paddings
|--------------------------------------------------------------------
 */

		$wp_customize->add_section(
			'fdwp_padding',
			[
				'title' => 'Abstände',
				'panel' => 'fdwp_panel',
			]
		);

		$wp_customize->add_setting(
			'fdwp-customizer[padding][cell_bt_full]',
			[
				'default' => '12px',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[padding][cell_bt_full]',
				[
					'type'               => 'number',
					'description_hidden' => false,
					'description'        => 'Default: 12px nach oben und unten',
					'label'              => 'Zellenabstände (Full)',
					'section'            => 'fdwp_padding',
					'settings'           => 'fdwp-customizer[padding][cell_bt_full]',
				]
			)
		);

		$wp_customize->add_setting(
			'fdwp-customizer[padding][cell_lr_full]',
			[
				'default' => '12px',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[padding][cell_lr_full]',
				[
					'type'               => 'number',
					'description_hidden' => false,
					'description'        => 'Default: 12px nach links und rechts',
					'label'              => 'Zellenabstände (Full)',
					'section'            => 'fdwp_padding',
					'settings'           => 'fdwp-customizer[padding][cell_lr_full]',
				]
			)
		);

		// mini (default)
		$wp_customize->add_setting(
			'fdwp-customizer[padding][cell_bt_mini]',
			[
				'default' => '6px',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[padding][cell_bt_mini]',
				[
					'type'               => 'number',
					'description_hidden' => false,
					'description'        => 'Default: 6px nach oben und unten',
					'label'              => 'Zellenabstände (Mini)',
					'section'            => 'fdwp_padding',
					'settings'           => 'fdwp-customizer[padding][cell_bt_mini]',
				]
			)
		);

		$wp_customize->add_setting(
			'fdwp-customizer[padding][cell_lr_mini]',
			[
				'default' => '12px',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[padding][cell_lr_mini]',
				[
					'type'               => 'number',
					'description_hidden' => false,
					'description'        => 'Default: 12px nach links und rechts',
					'label'              => 'Zellenabstände (Mini)',
					'section'            => 'fdwp_padding',
					'settings'           => 'fdwp-customizer[padding][cell_lr_mini]',
				]
			)
		);

		/**
		 * Border
		 */

		$wp_customize->add_section(
			'fdwp_border',
			[
				'title' => 'Border',
				'panel' => 'fdwp_panel',
			]
		);

		$wp_customize->add_setting(
			'fdwp-customizer[border][height]',
			[
				'default' => '1',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[border][height]',
				[
					'type'               => 'number',
					'description_hidden' => false,
					'description'        => 'Default: 1, 0 um border zu deaktivieren',
					'label'              => 'Border unter Tabellenzeile',
					'section'            => 'fdwp_border',
					'settings'           => 'fdwp-customizer[border][height]',
				]
			)
		);

		$wp_customize->add_setting(
			'fdwp-customizer[border][color]',
			[
				'default' => '#b2b2b2',
				'type'    => 'option',
			]
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'fdwp-customizer[border][color]',
				[
					'description_hidden' => false,
					'description'        => 'Default: #b2b2b2',
					'label'              => 'Border-Color',
					'section'            => 'fdwp_border',
					'settings'           => 'fdwp-customizer[border][color]',
				]
			)
		);

	}

}