<?php
namespace Crt\Fussballdaten\src\api;

use Crt\Fussballdaten\Fussballdaten;

/**
 * Class MatchdayRestRoute
 * @package Crt\Fussballdaten\src\api
 */
class MatchdayRestRoute extends FussballdatenRestRoute {

	/**
	 * Register the routes for the objects of the controller.
	 */
	public function __construct() {
		$this->base = 'competition/matchday';
		register_rest_route($this->namespace, '/' . $this->base, array(
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array($this, 'get_data'),
				'permission_callback' => array($this, 'get_items_permissions_check'),
				'args'                => array(),
			),
		));

		register_rest_route($this->namespace, '/' . $this->base . '/(?P<id>[\d]+)', array(
			array(
				'methods'             => \WP_REST_Server::READABLE,
				'callback'            => array($this, 'get_single'),
				'permission_callback' => array($this, 'get_item_permissions_check'),
				'args'                => array(
					'context' => $this->get_context_param(array('default' => 'view')),
				),
			),
		));
	}

	/**
	 * Get a collection of items
	 *
	 * @param \WP_REST_Request $request Full data about the request.
	 *
	 * @return \WP_Error|WP_REST_Response
	 */
	public function get_data($request) {
		$data = $this->prepare_response_for_collection(Fussballdaten::get_instance()->getCache()['matchdays']);
		return new \WP_REST_Response($data, 200);
	}

	/**
	 * Get a collection of items
	 *
	 * @param \WP_REST_Request $request Full data about the request.
	 *
	 * @return \WP_Error|WP_REST_Response
	 */
	public function get_single($request) {
		$id        = (int) $request['id'];
		$matchdays = Fussballdaten::get_instance()->getCache()['matchdays'];

		if (empty($id) || !isset($matchdays['fixtures'][$id])) {
			return new \WP_Error('invalid_matchday_id', __('Invalid Matchday id.'), array('status' => 404));
		}

		$response = array(
			'lastUpdate' => $matchdays['lastUpdate'],
			'fixture'    => $matchdays['fixtures'][$id],
		);

		$data = $this->prepare_response_for_collection($response);

		return new \WP_REST_Response($data, 200);
	}
}
