<?php
namespace Crt\Fussballdaten\src\page;

use Crt\Fussballdaten\Fussballdaten;
use Crt\Fussballdaten\src\service\CacheService;
use Crt\Fussballdaten\src\Singleton;

/**
 * Class SettingPage
 * @package Volbach\FootballData\src\page
 */
class SettingPage extends Singleton {

	/** @var mixed|void */
	private $options;

	/** @var mixed|void */
	private $cache;

	/**
	 * Option key, and option page slug.
	 *
	 * @var    string
	 * @since  0.1.0
	 */
	protected static $key = 'fussballdaten_settings';

	protected static $version = '1.1.1';

	public $wp_version;

	/**
	 * Init
	 */
	public function init() {

		global $wp_version;

		$this->wp_version = $wp_version;

		add_action('admin_menu', array($this, 'add_admin_page'));
		add_action('admin_enqueue_scripts', array($this, 'jquery_tabs'));
		add_action('admin_enqueue_scripts', array($this, 'load_select2_scripts'));
		add_action('admin_head', array($this, 'plugin_styles'));
		add_action('admin_init', array($this, 'admin_init'));

		add_action('wp_ajax_nopriv_fussballdaten_update_cache', array($this, 'ajax_update_cache'));
		add_action('wp_ajax_fussballdaten_update_cache', array($this, 'ajax_update_cache'));

	}

	public function jquery_tabs($hook) {
		// Only add to the edit.php admin page.
		// See WP docs.
		//if ('edit.php' !== $hook) {
		//	return;
		//}
		wp_enqueue_script('my_jquery_script', plugin_dir_url(__FILE__) . 'js/jtabs.js');
	}

	public function plugin_styles($hook) {
		// Only add to the edit.php admin page.
		// See WP docs.
		//if ('edit.php' !== $hook) {
		//	return;
		//}
		wp_enqueue_style('admin_styles', plugin_dir_url(__FILE__) . 'css/style.css');
	}

	/**
	 * admin init
	 */
	public function admin_init() {
		register_setting('fussballdaten_settings', 'fussballdaten_settings', array($this, 'options_validate'));

	}

	/**
	 * admin init
	 */
	public function ajax_update_cache() {
		CacheService::get_instance()->update_cache();

		header('Content-type: application/json');

		echo json_encode(array('success' => true));

		die();
	}

	/**
	 * @param $input
	 *
	 * @return mixed
	 */
	public function options_validate($input) {
		return $input;
	}

	function load_select2_scripts() {
		wp_enqueue_script('jquery');
		wp_enqueue_script('select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.13/js/select2.min.js', array('jquery'), '4.0.13', true);
		wp_enqueue_style('select2', 'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.13/css/select2.min.css', array(), '4.0.13', 'all');
	}

	function custom_auth_endpoint() {
		// Führe die Authentifizierung durch und gib ein Token zurück
		if ($this->options['apiToken']) {
			$token = $this->options['apiToken'];
			wp_send_json($token);
		}
	}

	/**
	 * add menu link
	 */
	public function add_admin_page() {
		//add_options_page('Settings Page Test', 'Settings Test', 'manage_options', 'misha_test_page', 'misha_test_page_callback');

		add_menu_page(
			'Fussballdaten Settings', // Seitentitel
			'Fussballdaten', // Menütitel
			'manage_options', // Benutzerberechtigung
			'fussballdaten_settings', // Seitenslug (URL)
			array($this, 'admin_template'),
			//'dashicons-admin-generic' // Icon (optional)
			plugins_url('fussballdaten/dist/img/logo.png'),
		);
	}

	/**
	 * Display options page
	 */
	public function admin_template() {
		// check the capability
		if (!current_user_can('manage_options')) {
			wp_die('You do not have sufficient permissions to access this page.');
		}

		add_action('wp_ajax_custom_auth', 'custom_auth_endpoint');
		add_action('wp_ajax_nopriv_custom_auth', 'custom_auth_endpoint');

		$this->options = Fussballdaten::get_instance()->getOptions();
		$this->cache   = Fussballdaten::get_instance()->getCache();

		//Get all Competitions if Api Token exist
		if ($this->options['apiToken']) {
			// Cache all matchdays

			$curl = curl_init();

			curl_setopt_array($curl, array(
				CURLOPT_URL            => "https://api.fussballdaten.de/wp/competition/",
				CURLOPT_RETURNTRANSFER => true,
				CURLOPT_ENCODING       => "",
				CURLOPT_MAXREDIRS      => 10,
				CURLOPT_TIMEOUT        => 30,
				CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
				CURLOPT_CUSTOMREQUEST  => "GET",
				CURLOPT_HTTPHEADER     => array(
					"Cache-Control: no-cache",
					"Authorization: Bearer " . $this->options['apiToken'],
				),
			));

			$response = curl_exec($curl);
			$response = json_decode($response, true);
			//\var_dump($response);
			if (isset($response['status'])) {
				$allCompetitions = ['error' => $response['message']];
			} else {
				$allCompetitions = $response['data'];
			}
			//\var_dump($allCompetitions);
			curl_close($curl);

			$alls = [];

			if (!empty($this->options['teamId'])) {
				$teamIds = implode(',', $this->options['teamId']);

				$curl = curl_init();

				curl_setopt_array($curl, array(
					CURLOPT_URL            => "https://api.fussballdaten.de/wp/team/?ids=" . $teamIds,
					CURLOPT_RETURNTRANSFER => true,
					CURLOPT_ENCODING       => "",
					CURLOPT_MAXREDIRS      => 10,
					CURLOPT_TIMEOUT        => 30,
					CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
					CURLOPT_CUSTOMREQUEST  => "GET",
					CURLOPT_HTTPHEADER     => array(
						"Cache-Control: no-cache",
						"Authorization: Bearer " . $this->options['apiToken'],
					),
				));

				$response = curl_exec($curl);
				$response = json_decode($response, true);
				curl_close($curl);

				if (isset($response['status'])) {
					$allTeams = ['error' => $response['message']];
				} else {
					$allTeams = $response['data'];
				}
			}

		}

//Get the active tab from the $_GET param
		$default_tab = null;
		$tab         = isset($_GET['tab']) ? $_GET['tab'] : $default_tab;

		?>

		<div class="wrap">
			<div id="icon-options-general" class="icon32"><br></div>
			<img src="<?=plugins_url('fussballdaten/dist/img/fd_logo_opt.webp')?>" style="height:40px;margin-left:5px;" />

			<br/>
<?php
if (defined('DISABLE_WP_CRON') && DISABLE_WP_CRON) {
			?>
								<div class="notice notice-error" style="margin:10px 0 10px 8px;">
									<p><b>Wordpress Cron nicht aktiv</b></p>
									<p>
										Bitte aktiviere Wordpress Cron, ansonsten werden die Daten des Plugins nicht fortlaufen aktualisiert.
									</p>
								</div>
<?php
}
		?>

			<form action="options.php" method="post" >
<?php
settings_fields('fussballdaten_settings');
		do_settings_sections('options');
		?>

				<nav class="nav-tab-wrapper">
					<a href="?page=<?=self::$key?>" data-id-target="#settings" class="nav-tab <?php if ($tab === null): ?>nav-tab-active<?php endif;?>">Einstellungen</a>
					<a href="?page=<?=self::$key?>&tab=shortcodes"  data-id-target="#shortcodes" class="nav-tab <?php if ($tab === 'shortcodes'): ?>nav-tab-active<?php endif;?>">Shortcodes</a>
					<a href="?page=<?=self::$key?>&tab=styling"  data-id-target="#styling" class="nav-tab <?php if ($tab === 'styling'): ?>nav-tab-active<?php endif;?>">Styling</a>
					<a href="?page=<?=self::$key?>&tab=support"  data-id-target="#support" class="nav-tab <?php if ($tab === 'support'): ?>nav-tab-active<?php endif;?>">Support</a>
				</nav>

				<div id="settings" class="tab-content active">

				<table class="form-table">
					<tbody>
					<tr valign="top">
						<th scope="row">
							Api Token<br>
						</th>
						<td>
							<input type="text" class="regular-text ltr" name="fussballdaten_settings[apiToken]" value="<?php echo (isset($this->options['apiToken'])) ? $this->options['apiToken'] : ''; ?>" />
							<br><small>Um ein gültiges Api Token zu erhalten, bitte an <a href="mailto:technik@fussballdaten.de">technik@fussballdaten.de</a> wenden.</small>
						</td>
					</tr>
					<?php if (isset($allCompetitions) && !isset($allCompetitions['error'])) {
			?>
						<tr valign="top">
							<th scope="row">
								Wettbewerbe<br>
							</th>
							<td>
								<select name="fussballdaten_settings[competitionId][]" multiple="true" style="width:350px;height:200px">
									<?php foreach ($allCompetitions as $competition) {?>
										<option value="<?php echo $competition['id']; ?>"<?php echo (isset($this->options['competitionId']) && in_array($competition['id'], $this->options['competitionId'])) ? ' selected' : ''; ?>><?php echo $competition['land'] . ' - ' . $competition['name']; ?></option>
									<?php }?>
								</select>
							</td>
						</tr>
						<tr>
							<th scope="row">
								Verein<br>
							</th>
							<td>
								<select id="ajax-search" name="fussballdaten_settings[teamId][]" multiple="multiple" style="width:350px">								</select>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row">
								Cache<br />
							</th>
							<td>
<?php
if (!isset($allCompetitions['error'])) {
				?>
				<input type="button" class="button button-default" id="update_fussballdaten_cache" value="Manuelles Update">
				<?php
}?>
							</td>
						</tr>
					<?php
} else if (isset($allCompetitions['error'])) {?>
						<tr valign="top">
							<th colspan="2">
								<div class="notice notice-error">
									<p><?=$allCompetitions['error']?></p>
									<small>Bitte an <a href="mailto:technik@fussballdaten.de">technik@fussballdaten.de</a> wenden.</small><br /><br />
								</div>
							</th>
						</tr>
					<?php }?>
					</tbody>
				</table>


				<p>				<?php
submit_button();
		?></p>

				</div>
				<div id="shortcodes"  class="tab-content">
<?php
if (isset($this->options['apiToken']) && !empty($this->options['apiToken'])) {
			if (!isset($allCompetitions['error'])) {
				if (!empty($this->options['competitionId'])) {
					?>
				<table class="form-table">
									<tbody>
						<tr>
							<td colspan="2">
								Jeder Shortcode hat optionale Attribute:<br />
								<br />
								<code>
									display = "", "mini" oder "full" (default "mini");<br />
									headline = "" oder frei definierbar;<br />
									links = "" oder "inline" (default = "")
								</code>
								<br /><br />
								Um die Default-Headline zu entfernen also einfach in dem Shortcode <code>headline=""</code> angeben. Jeder Shortcode ist per default für die Sidebar ausgelegt. Um dies zu ändern muss das Attribut display definiert werden: <code>display="full"</code>. Jeder Shortcode generiert 3 externe Links zu fussballdaten.de. Die Links werden unter dem Widget dargestellt. Diese sind per default untereinander. Um die Links nebeneinander darzustellen muss das Attribut links definiert werden: <code>links="inline"</code>
							</td>
						</tr>
					</tbody>
				</table>

				<table class="form-table">

					<thead>
						<tr>
							<th colspan="2" style="border-bottom:solid 2px #ddd;font-size:18px;text-transform:uppercase;">Wettbewerbe:</th>
						</tr>
					</thead>
					<tbody>
<?php

					foreach ($allCompetitions as $competition) {

						if (in_array($competition['id'], $this->options['competitionId'])) {
							?>
					<tr>
						<th colspan="2"><b style="font-size:14px;border-bottom:1px solid;padding-bottom:2px;"><?=$competition['name']?></b></th>
					</tr>
					<tr valign="top">
						<th scope="row">
							Tabelle<br>
						</th>
						<td>
							<p style="display:none">Letzte Aktualisierung: <?php echo (isset($this->cache['table']['lastUpdate']) ? $this->cache['table']['lastUpdate'] : '-'); ?></p>
							<p>Default Headline: Tabelle</p>
							<br />
							<p>Shortcode: <code>[fussballdaten type="table" competition="<?=$competition['id']?>"]</code></p>
							<p>Darstellung ohne Siege, Niederlagen, Unentschieden und Torverhältnis - optimal für die Sidebar. Vereine haben keine Wappen.</p>
							<br />
							<p>Shortcode: <code>[fussballdaten type="table" competition="<?=$competition['id']?>" display="full"]</code></p>
							<p>Darstellung mit Siegen, Niederlagen, Unentschieden und Torverhältnis - optimal für den Content-Bereich. Vereine werden mit Wappen dargestellt.</p>
							<br />
							<p>Shortcode: <code>[fussballdaten type="table" competition="<?=$competition['id']?>" display="full" links="inline"]</code></p>
							<p>Links unter der Tabelle werden nebeneinander dargestellt (Default: untereinander)</p>
						</td>
					</tr>
					<tr valign="top">
						<th scope="row">
							Spieltage<br>
						</th>
						<td>
							<p style="display:none">Letzte Aktualisierung: <?php echo (isset($this->cache['matchdays']['lastUpdate']) ? $this->cache['matchdays']['lastUpdate'] : '-'); ?></p>
							<p>Default Headline: Spieltag</p>
							<br />
							<p>Shortcode: <code>[fussballdaten type="current_matchday" competition="<?=$competition['id']?>"]</code></p>
							<p>Kurze Vereinsnamen ohne Wappen - optimal für die Sidebar</p>
							<br />
							<p>Shortcode: <code>[fussballdaten type="current_matchday" competition="<?=$competition['id']?>" display="full"]</code></p>
							<p>Lange Vereinsnamen mit Wappen - optimal für den Content-Bereich</p>
							<br />
							<p>Shortcode: <code>[fussballdaten type="current_matchday" competition="<?=$competition['id']?>" display="full" links="inline"]</code></p>
							<p>Links unter der Tabelle werden nebeneinander dargestellt (Default: untereinander)</p>
						</td>
					</tr>
                	<!--
					<tr valign="top">
                    	<th scope="row">Verein hervorheben:</th>
                    	<td>
                        	<select name="btp_selected_club" id="btp_selected_club" <?php //echo $btp_api_key_valid ? '' : 'disabled'; ?>>
    	                    </select>
							<p>Shortcode: [fussballdaten type="current_matchday" competition="<?//=$competition['id']?>"]</p>
        	            </td>
            	    </tr>
			-->
<?php
}
					}
					?>
					</tbody>
<?php
if (!empty($allTeams)) {

						?>

					<thead>
						<tr>
							<th colspan="2" style="border-bottom:solid 2px #ddd;font-size:18px;text-transform:uppercase;">Vereine:</th>
						</tr>
					</thead>
<?php

						foreach ($allTeams as $team) {
							?>
					<tr>
						<th colspan="2"><b style="font-size:14px;border-bottom:1px solid;padding-bottom:2px;"><?=$team['text']?></b></th>
					</tr>
					<tr valign="top">
						<th scope="row">
							Spielplan<br>
						</th>
						<td>
							<p>Default Headline: Spielplan</p>
							<br />
							<p>Shortcode: <code>[fussballdaten type="schedule" team="<?=$team['id']?>"]</code></p>
							<p>Stellt den kompletten Spielplan der aktuellen Saison dar.</p>
							<br />
							<p>Shortcode: <code>[fussballdaten type="schedule" team="<?=$team['id']?>" display="full"]</code></p>
							<p>Darstellung mit Wappen, Datum und Wettbewerb.</p>
						</td>
					</tr>
					<tr valign="top">
						<th scope="row">
							Nächste Spiele<br>
						</th>
						<td>
							<p>Default Headline: Die nächsten $next Spiele</p>
							<br />

							<p>Shortcode: <code>[fussballdaten type="schedule" team="<?=$team['id']?>" display="full" next="3"]</code></p>
							<p>Stellt die nächsten 3 (next="3") Spiele der aktuellen Saison ab dem aktuellen Datum dar.</p>
							<br />
							<p>Shortcode: <code>[fussballdaten type="schedule" team="<?=$team['id']?>" display="full" next="5" datum="2024-03-15"]</code></p>
							<p>Stellt die nächsten 5 (next="5") Spiele der aktuellen Saison ab dem gewähltem Datum (date="2024-03-15") dar.</p>
						</td>
					</tr>

<?php
}
					}
					?>
				</table>




<?php
} else {
					if (isset($allCompetitions['error'])) {
						?>
								<div class="notice notice-error">
									<p><?=$allCompetitions['error']?></p>
									<small>Bitte an <a href="mailto:technik@fussballdaten.de">technik@fussballdaten.de</a> wenden.</small><br /><br />
								</div>
<?php
}
				}
			}
		}
		if (isset($this->options['apiToken']) && empty($this->options['apiToken'])) {
			?>
								<div class="notice notice-error" style="margin:10px 0 10px 8px;">
									<p><b>Kein gültiges API Token</b></p>
									<p>
										Nach Eingabe des gültigen API Tokens werden die Shortcodes zu den ausgewählten Wettbewerben dargestellt.
									</p>
								</div>
<?php
} else {

			if (isset($allCompetitions['error'])) {
				?>
								<div class="notice notice-error">
									<p><?=$allCompetitions['error']?></p>
									<small>Bitte an <a href="mailto:technik@fussballdaten.de">technik@fussballdaten.de</a> wenden.</small><br /><br />
								</div>
<?php

			} else {
				if (empty($this->options['competitionId'])) {
					?>
								<div class="notice notice-error" style="margin:10px 0 10px 8px;">
									<p><b>Keine Wettbewerbe definiert</b></p>
									<p>
										Bitte wähle unter Einstellungen mindestens einen Wettbewerb aus, um die die dazugehörigen Shortcodes anzeigen zu lassen.
									</p>
								</div>
<?php
}
			}
		}
		?>
				</div>
				<div id="styling"  class="tab-content">
					<p>Einige Anzeigeeinstellungen des Plugins lassen sich über den Customizer bearbeiten. </p>
					<p><a class="button button-secondary ml-auto" target="_blank" href="<?php echo esc_url(admin_url('customize.php?autofocus[panel]=fdwp_panel')); ?>">Customizer öffnen</a></p>
				</div>
				<div id="support" class="tab-content">

				<div class="postbox" style="margin-top:20px;">
					<div class="inside">
						<p>
						<?php echo 'Wenn Du einen Fehler findest, Hilfe brauchst oder eine Funktion vorschlagen möchtest, kontaktiere bitte '; ?>:
						<br>
						- <a href="mailto:technik@fussballdaten.de"><?php echo 'technik@fussballdaten.de'; ?></a>
						<br><br>
						Gib bitte immer Deine System-Informationen an.
						</p>

						<h4><?php echo 'Deine System-Informationen'; ?></h4>

						<ul>
							<li>============================================</li>
							<li>
								<b>Plugin Version:</b> Fussballdaten WP <?php echo self::$version; ?>
							</li>

							<li>
								<b>WordPress Version:</b> <?php echo $this->wp_version; ?>
							</li>
							<li>
								<b>Aktiver Theme:</b> <?php echo get_template(); ?>
							</li>

							<li>
								<b>Server Time:</b> <?php echo date_default_timezone_get(); ?>
							</li>

							<li>
								<b>WP Time:</b> <?php echo get_option('timezone_string'); ?>
							</li>

							<li>
								<b>Current Date:</b> <?php echo date_i18n('Y-m-d'); ?>
							</li>

							<li>
								<b>Site Locale:</b> <?php echo get_locale(); ?>
							</li>


							<li>
								<b>PHP version:</b> <?php echo phpversion(); ?>
							</li>
							<li>
								<b><?php echo 'Active Plugins'; ?>:</b>
								<?php
foreach (get_option('active_plugins') as $value) {
			$string = explode('/', $value);
			echo '<br>--- ' . esc_html($string[0]);
		}
		?>
							</li>
							<li>============================================</li>
						</ul>
					</div>
				</div>


				</div>

				<hr>

				<div style="clear: both;"></div>
			</form>
		</div>

		<script>
            jQuery(document).ready(function($){
                $('#update_fussballdaten_cache').on('click', function() {
                    $.ajax({url: "<?php echo admin_url('admin-ajax.php') ?>?action=fussballdaten_update_cache", success: function(result){
                        alert('Cache wurde erfolgreich geupdatet!');
                    }});
                });
            });

			 var preselectedTeams = <?php echo json_encode($allTeams); ?>;


			jQuery(document).ready(function($) {

				$('#ajax-search').select2({
					ajax: {
						url: 'https://api.fussballdaten.de/wp/team/search', // Die WordPress AJAX URL
						dataType: 'json',
						contentType: "application/json",
						delay: 250,
						headers: {
							"Authorization": "Bearer <?=$this->options['apiToken']?>",
						},
						processResults: function(data) {
							return {
								results: data
							};
						},
						cache: true
					},
					minimumInputLength: 3, // Mindestanzahl an Zeichen, bevor die Suche ausgelöst wird,
					maximumSelectionLength: 3,
					allowClear: true,
				});
			});

  			preselectedTeams.forEach(function(team) {
    			// Ein neues Optionselement erstellen und hinzufügen
    			var newOption = new Option(team.text, team.id, true, true);
    			jQuery('#ajax-search').append(newOption).trigger('change');
  			});
		</script>
		<?php
}
}
