<?php
namespace Crt\Fussballdaten\src\service;

use Crt\Fussballdaten\Fussballdaten;
use Crt\Fussballdaten\src\Singleton;

/**
 * Class CacheService
 * @package Crt\Fussballdaten\src\service
 *
 * Todo: clean up api version switch
 */
class CacheService extends Singleton {

	/** @var mixed|void */
	private $options;

	/** @var mixed|void */
	private $cache;

	/**
	 * admin init
	 */
	public function update_cache() {

		$this->options = Fussballdaten::get_instance()->getOptions();
		$this->cache   = Fussballdaten::get_instance()->getCache();

		if ($this->options['apiToken'] && $this->options['competitionId']) {
			$this->cache_table();
			$this->cache_competition();
			$this->cache_matchdays();
			$this->cache_schedule();
			Fussballdaten::get_instance()->setCache($this->cache);
		}
	}

/**
 * Update Cache - table
 */
	public function cache_table() {

		foreach ($this->options['competitionId'] as $id) {

			$response = $this->_callApi('competition/standings?id=' . $id);
			//var_dump($response);
			// Save
			if (!isset($response['error']) && isset($response['items'])) {
				$this->cache['table'][$id] = [];

				// Prepair Standing
				$prepairedStanding = [];
				foreach ($response['items'] as $r) {
					$prepairedStanding[] = [
						'position'       => (int) $r['rank'],
						'previous'       => (int) $r['rank_previous'],
						'rankDifference' => (string) $r['rank_diff'],
						'teamName'       => (string) $r['teamName'],
						'teamNameShort'  => (string) $r['teamNameShort'],
						'teamLogo'       => (string) $r['logo'],
						'playedGames'    => (int) $r['number_games'],
						'points'         => (int) $r['number_points'],
						'goals'          => (int) $r['goals_plus'],
						'goalsAgainst'   => (int) $r['goals_minus'],
						'goalDifference' => (string) $r['goals_diff'],
						'wins'           => (int) $r['won'],
						'draws'          => (int) $r['draw'],
						'losses'         => (int) $r['loss'],
					];
				}

				$this->cache['table'][$id] = [
					'lastUpdate'    => date(\DateTime::ATOM, time()),
					'leagueCaption' => $response['competition']['name'],
					'matchday'      => $response['matchday']['currentMatchday'],
					'standing'      => $prepairedStanding,
				];

				if (isset($response['links'])) {
					$this->cache['table'][$id]['links'] = $response['links'];
				} else {
					$this->cache['table'][$id]['links'] = null;
				}

			}
		}

		$this->cache['table']['lastUpdate'] = date(\DateTime::ATOM, time());

	}

	/**
	 * Update Cache - competitions
	 */
	public function cache_competition() {

		foreach ($this->options['competitionId'] as $id) {

			$response = $this->_callApi('competition/?id=' . $id);
			// Save
			if (!isset($response['error']) && isset($response)) {
				$this->cache['competition'][$id] = [];
				$this->cache['competition'][$id] = [
					'lastUpdate'      => date(\DateTime::ATOM, time()),
					'id'              => $response['data']['id'],
					'caption'         => $response['data']['land'],
					'name'            => $response['data']['name'],
					//'year' => $response['year'],
					'currentMatchday' => $response['data']['matchday']['currentMatchday'],
					//'numberOfMatchdays' => $response['numberOfMatchdays'],
					//'numberOfTeams' => $response['numberOfTeams'],
					//'numberOfGames' => $response['numberOfGames'],
				];

			}
		}
		//\var_dump($this->cache['competition']);
	}

	/**
	 * Update Cache - cache matches
	 */
	public function cache_matchdays() {
		// Cache all matchdays

		foreach ($this->options['competitionId'] as $id) {

			$response = $this->_callApi('competition/matchday?id=' . $id);

			if (!isset($response['error']) && isset($response['items'])) {
				$this->cache['matchdays'][$id] = [];

				$matchDay = ($response['matchday']['selectedMatchday'] > 0) ? $response['matchday']['selectedMatchday'] : $response['matchday']['currentMatchday'];

				// prepair fixtures
				$prepairedFixtures = [];
				foreach ($response['items'] as $fixture) {

					$htHome = null;
					$htAway = null;

					if (!empty($fixture['match']['halftime'])) {
						$tmp    = explode(':', $fixture['halftime']);
						$htHome = (int) $tmp[0];
						$htAway = (int) $tmp[1];
					}

					$prepairedFixtures[$matchDay][] = [
						'date'          => $fixture['match']['date'],
						'time'          => $fixture['match']['time'],
						'status'        => $fixture['match']['status'],
						'homeTeamName'  => $fixture['home']['teamName'],
						'homeTeamShort' => $fixture['home']['teamNameShort'],
						'homeTeamLogo'  => $fixture['home']['logo'],
						'awayTeamName'  => $fixture['away']['teamName'],
						'awayTeamShort' => $fixture['away']['teamNameShort'],
						'awayTeamLogo'  => $fixture['away']['logo'],
						'result'        => [
							'goalsHomeTeam' => (is_null($fixture['results']['goals_home'])) ? '' : $fixture['results']['goals_home'],
							'goalsAwayTeam' => (is_null($fixture['results']['goals_away'])) ? '' : $fixture['results']['goals_away'],
							'halfTime'      => [
								'goalsHomeTeam' => (is_null($htHome)) ? '' : $htHome,
								'goalsAwayTeam' => (is_null($htAway)) ? '' : $htAway,
							],
						],
						'links'         => [
							'compare' => $fixture['links']['compare'],
							'match'   => $fixture['links']['match'],
						],
					];
				}

				$this->cache['matchdays'][$id] = [
					'lastUpdate' => date(\DateTime::ATOM, time()),
					'fixtures'   => $prepairedFixtures,
					'matchday'   => $response['matchday'],
				];

				if (isset($response['links'])) {
					$this->cache['matchdays'][$id]['links'] = $response['links'];
				} else {
					$this->cache['matchdays'][$id]['links'] = null;
				}

			}

		}

	}

	/**
	 * Update Cache - cache matches
	 */
	public function cache_schedule() {
		// Cache all matchdays

		//foreach ($this->options['teamId'] as $id) {
		if (isset($this->options['teamId'])) {

			foreach ($this->options['teamId'] as $id) {

				$response = $this->_callApi('team/schedule?id=' . $id);

				if (!isset($response['error']) && isset($response['items'])) {
					$this->cache['schedule'][$id] = [];

					//$matchDay = ($response['matchday']['selectedMatchday'] > 0) ? $response['matchday']['selectedMatchday'] : $response['matchday']['currentMatchday'];

					// prepair fixtures
					$prepairedFixtures = [];
					foreach ($response['items'] as $fixture) {

						$htHome = null;
						$htAway = null;

						if (!empty($fixture['match']['halftime'])) {
							$tmp    = explode(':', $fixture['halftime']);
							$htHome = (int) $tmp[0];
							$htAway = (int) $tmp[1];
						}

						$prepairedFixtures[] = [
							'date'          => $fixture['match']['date'],
							'time'          => $fixture['match']['time'],
							'status'        => $fixture['match']['status'],
							'matchday'      => $fixture['match']['matchday'],
							'round'         => $fixture['match']['round'],
							'competition'   => $fixture['match']['competition'],
							'homeTeamName'  => $fixture['home']['teamName'],
							'homeTeamShort' => $fixture['home']['teamNameShort'],
							'homeTeamLogo'  => $fixture['home']['logo'],
							'awayTeamName'  => $fixture['away']['teamName'],
							'awayTeamShort' => $fixture['away']['teamNameShort'],
							'awayTeamLogo'  => $fixture['away']['logo'],

							'result'        => [
								'goalsHomeTeam' => (is_null($fixture['results']['goals_home'])) ? '' : $fixture['results']['goals_home'],
								'goalsAwayTeam' => (is_null($fixture['results']['goals_away'])) ? '' : $fixture['results']['goals_away'],
								'halfTime'      => [
									'goalsHomeTeam' => (is_null($htHome)) ? '' : $htHome,
									'goalsAwayTeam' => (is_null($htAway)) ? '' : $htAway,
								],
								'penalties'     => $fixture['results']['goals_home'],
								'overtime'      => $fixture['results']['overtime'],
							],
							'links'         => [
								'compare' => $fixture['links']['compare'],
								'match'   => $fixture['links']['match'],
							],
						];

					}

					$this->cache['schedule'][$id] = [
						'lastUpdate' => date(\DateTime::ATOM, time()),
						'fixtures'   => $prepairedFixtures,
					];

					if (isset($response['links'])) {
						$this->cache['schedule'][$id]['links'] = $response['links'];
					} else {
						$this->cache['schedule'][$id]['links'] = null;
					}

				}
			}

		}

	}

	private function _callApi($route) {

		$curl = curl_init();

		curl_setopt_array($curl, array(
			CURLOPT_URL            => Fussballdaten::API_URI . $route,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_ENCODING       => "",
			CURLOPT_MAXREDIRS      => 10,
			CURLOPT_TIMEOUT        => 30,
			CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
			CURLOPT_CUSTOMREQUEST  => "GET",
			CURLOPT_HTTPHEADER     => array(
				"Cache-Control: no-cache",
				"Authorization: Bearer " . $this->options['apiToken'],
			),
		));

		$response = curl_exec($curl);
		$response = json_decode($response, true);

		//var_dump(Fussballdaten::API_URI . $route);

		curl_close($curl);
		return $response;
	}

}

?>