<?php
namespace Crt\Fussballdaten\src\shortcode;

use Crt\Fussballdaten\Fussballdaten;
use Crt\Fussballdaten\src\Singleton;

class GeneralShortcode extends Singleton {

	private $cache;

	/**
	 * FussballdatenShortcode constructor.
	 */
	public function init() {
		add_action('init', array($this, 'register_shortcodes'), 0);
	}

	/**
	 * Register Shortcode
	 */
	public function register_shortcodes() {
		add_shortcode('fussballdaten', array($this, 'shortcode'));
	}

	/**
	 * General Shortcode
	 *
	 * @param $atts
	 * @param null $content
	 *
	 * @example [fussballdaten type="name_of_type" lazy-load=false competition=$id]
	 *
	 * @return string
	 */
	public function shortcode($atts, $content = null) {
		$output = '';

		// Register Scripts
		wp_enqueue_style('fussballdaten', plugins_url('../../dist/css/style.css', __FILE__), false, true);
		wp_enqueue_script('fussballdaten', plugins_url('../../dist/js/script.min.js', __FILE__), array('jquery'), false, true);

		// Output
		if (isset($atts['type']) && '' == $atts['type']) {
			return 'Type is required';
		} elseif (isset($atts['competition']) && '' == $atts['competition']) {
			return 'Competition is required';
		} else {

			$customCss = $this->_setCustomizerStyles();

			if (isset($atts['lazy-load']) && $atts['lazy-load']) {
				$output .= '<div class="fdwp-table"></div>';
				$output .= '<div class="loader-wrapper"><div class="loader"></div></div>';
			} else {
				if ('current_matchday' == $atts['type']) {
					$output .= $this->getHtmlMatchday($atts, $customCss);
				} elseif ('table' == $atts['type']) {
					$output .= $this->getHtmlTable($atts, $customCss);
				} elseif ('schedule' == $atts['type']) {
					$output .= $this->getHtmlSchedule($atts, $customCss);
				}
			}

		}

		return $output;
	}

	private function _setCustomizerStyles() {
		$css = get_option('fdwp-customizer');

		//print_r($css);

		$tableHead      = [];
		$tableRow       = [];
		$tableRowStripe = [];

		if (isset($css['colors'])) {

			if (isset($css['colors']['th'])) {
				$tableHead['color'] = $css['colors']['th'];
			}

			if (isset($css['colors']['bg_th'])) {
				$tableHead['background-color'] = $css['colors']['bg_th'];
			}

			if (isset($css['colors']['td'])) {
				$tableRow['color'] = $css['colors']['td'];
			}

			if (isset($css['colors']['bg_td'])) {
				$tableRow['background-color'] = $css['colors']['bg_td'];
			}

			if (isset($css['colors']['bg_td_stripe'])) {
				$tableRowStripe['background-color'] = $css['colors']['bg_td_stripe'];
			}

		}

		if (isset($css['font_sizes'])) {
			if (isset($css['font_sizes']['table_header_row']) && ($css['font_sizes']['table_header_row'] > 0)) {

				$tableHead['font-size'] = $css['font_sizes']['table_header_row'] . 'px';
			}
			if (isset($css['font_sizes']['table_row']) && ($css['font_sizes']['table_row'] > 0)) {
				$tableRow['font-size']       = $css['font_sizes']['table_row'] . 'px';
				$tableRowStripe['font-size'] = $css['font_sizes']['table_row'] . 'px';
			}

		}

		if (isset($css['border']['height'])) {
			if ($css['border']['height'] == 0) {
				$border = 0;
			} else {

				$border = $css['border']['height'] . 'px solid ';
				if (isset($css['border']['color'])) {
					$border .= $css['border']['color'];
				}
			}

			$tableHead['border-bottom']      = $border;
			$tableRow['border-bottom']       = $border;
			$tableRowStripe['border-bottom'] = $border;

		}

		return [
			'tableHead'      => self::mapArrayValues($tableHead),
			'tableRow'       => self::mapArrayValues($tableRow),
			'tableRowStripe' => self::mapArrayValues($tableRowStripe),
		];

	}

	private static function mapArrayValues($data) {
		return implode(';', array_map(
			function ($v, $k) {return sprintf("%s:%s", $k, $v);},
			$data,
			array_keys($data)
		));
	}

	private static function teamNameClass($rowcls, $teamName, $teamNameShort) {
		$clsLongName  = ($rowcls == 'full') ? 'name-visible long' : 'name-hidden long';
		$clsShortName = ($rowcls != 'full') ? 'name-visible short' : 'name-hidden short';

		return '<span class="' . $clsLongName . '">' . $teamName . '</span><span class="' . $clsShortName . '">' . $teamNameShort . '</span>';
	}

	public function getHtmlMatchday($atts, $css = []) {
		$cache = Fussballdaten::get_instance()->getCache()['matchdays'][$atts['competition']];

		//echo "<pre>";
		//print_r($cache);
		//echo "</pre>";

		$matchdays       = $cache['matchday'];
		$currentMatchday = ($matchdays['selectedMatchday'] > 0) ? $matchdays['selectedMatchday'] : $matchdays['currentMatchday'];

		if (!isset($atts['headline'])) {
			$output = '<div class="widget-title block-title"><h3>' . $currentMatchday . '. Spieltag</h3></div>';
		} elseif ($atts['headline'] == 'false' || empty($atts['headline'])) {
			$output = '';
		} else {
			$output = '<div class="widget-title block-title"><h3>' . $atts['headline'] . '</h3></div>';
		}

		$output .= '<div class="fdwp-wrapper">';
		$output .= '<div class="fdwp-table fdwp-matchday">';

		if (isset($cache['fixtures'][$currentMatchday])) {
			$date = '';

			if (isset($atts['display']) && $atts['display'] == 'full') {
				$rowcls = 'full';
			} else {
				$rowcls = 'mini';
			}

			foreach ($cache['fixtures'][$currentMatchday] as $k => $match) {
				$homeScore = '-';
				$awayScore = '-';
				$htmlClass = '';

				$tz      = new \DateTimeZone('Europe/Berlin');
				$dateTmp = new \DateTime($match['date']);
				$dateTmp->setTimezone($tz);

				$timeTmp = new \DateTime($match['time']);
				//$timeTmp->setTimezone($tz);

				if ($date != $match['date']) {

					$date = $match['date'];

					$output .= '<div class="row date" style="' . $css['tableHead'] . '">';
					$output .= '<div class="cell">' . $dateTmp->format('d.m.Y') . '</div>';
					$output .= '<div class="cell">&nbsp;</div>';
					$output .= '<div class="cell">&nbsp;</div>';
					$output .= '</div>';
				}

				$style = ($k % 2 == 0) ? $css['tableRow'] : $css['tableRowStripe'];

				$resultBox = '<a href="' . $match['links']['compare'] . '" target="_blank" title="' . $match['homeTeamName'] . ' vs ' . $match['awayTeamName'] . ' - Formvergleich auf fussballdaten.de"><span class="time">' . $timeTmp->format('H:i') . '</span></a>';
				//$resultBox = '<span class="time result">1:3</span>';

				if ($match['status'] != 'TIMED' && $match['result']['goalsHomeTeam'] !== '') {
					$homeScore = $match['result']['goalsHomeTeam'];
					$awayScore = $match['result']['goalsAwayTeam'];

					$resultBox = '<a href="' . $match['links']['match'] . '" target="_blank" title="' . $match['homeTeamName'] . ' vs ' . $match['awayTeamName'] . ' - Aufstellungen & Spieldaten auf fussballdaten.de"><span class="result">' . $homeScore . ':' . $awayScore . '</span></a>';
				}

				if ($match['status'] == 'FINISHED') {
					$htmlClass += ' finished';
				}

				$output .= '<div class="row ' . $rowcls . '" style="' . $style . '">';

				$homeLogo = '<img src="' . $match['homeTeamLogo'] . '" width="20" height="20" alt="Logo ' . $match['homeTeamName'] . '" />';
				if ($rowcls != 'full') {
					$homeLogo = '';
				}

				$output .= '<div class="cell"><div class="flex">' . self::teamNameClass($rowcls, $match['homeTeamName'], $match['homeTeamShort']) . $homeLogo . '</div></div>';
				$output .= '<div class="cell' . $htmlClass . '">' . $resultBox . '</div>';

				$awayLogo = '<img src="' . $match['awayTeamLogo'] . '" width="20" height="20" alt="Logo ' . $match['homeTeamName'] . '" />';
				if ($rowcls != 'full') {
					$awayLogo = '';
				}

				$output .= '<div class="cell"><div class="flex">' . $awayLogo . self::teamNameClass($rowcls, $match['awayTeamName'], $match['awayTeamShort']) . '</div></div>';
				$output .= '</div>';
			}
		}
		$output .= '</div>';
		$output .= '</div>';

		if (isset($cache['links'])) {
			$output .= $this->getHtmlLinks($cache['links'], $atts, $css);
		}

		return $output;
	}

	public static function validateDate($date, $format = 'Y-m-d') {
		$d = \DateTime::createFromFormat($format, $date);
		return $d && $d->format($format) == $date;
	}

	public static function skipMatch($next, $i, $dateStart, $matchDate) {
		$ts1 = strtotime($dateStart);
		$ts2 = strtotime($matchDate);

		if ($ts1 < $ts2) {
			return false;
		} else {
			return true;
		}
	}

	public function getHtmlSchedule($atts, $css = []) {

		$cache = Fussballdaten::get_instance()->getCache()['schedule'][$atts['team']];

		$schedule = $cache['fixtures'];

		$next      = false;
		$dateStart = date('Y-m-d');

		if (isset($atts['next']) && is_numeric($atts['next']) && $atts['next'] > 0) {
			$next = $atts['next'];
			if (isset($atts['date'])) {
				if ($dateTmp = self::validateDate($atts['date'])) {
					$dateStart = $atts['date'];
				}
			}
		}

		if (!isset($atts['headline'])) {
			$output = '<div class="widget-title block-title"><h3>Spielplan</h3></div>';
			if ($next !== false) {
				if ($next == 1) {
					$output = '<div class="widget-title block-title"><h3>Nächstes Spiel</h3></div>';
				} else {
					$output = '<div class="widget-title block-title"><h3>Nächste ' . $next . ' Spiele</h3></div>';
				}
			}
		} elseif ($atts['headline'] == 'false' || empty($atts['headline'])) {
			$output = '';
		} else {
			$output = '<div class="widget-title block-title"><h3>' . $atts['headline'] . '</h3></div>';
		}

		$output .= '<div class="fdwp-wrapper" style="box-shadow:none">';
		$output .= '<div class="fdwp-table fdwp-matchday fdwp-schedule">';

		$unterminiertInfo = 0;

		if (!empty($schedule)) {
			$date = '';

			if (isset($atts['display']) && $atts['display'] == 'full') {
				$rowcls = 'full';
			} else {
				$rowcls = 'mini';
			}

			$i = 0;

			foreach ($schedule as $k => $match) {

				if ($next !== false) {
					if ($i >= $next) {
						continue;
					} else {
						if (self::skipMatch($next, $i, $dateStart, $match['date']) === true) {
							continue;
						} else {
							$i += 1;
						}
					}
				}

				$homeScore = '-';
				$awayScore = '-';
				$htmlClass = '';

				if ($date != $match['date']) {
					//$tz      = new \DateTimeZone('Europe/Berlin');
					$dateTmp = new \DateTime($match['date']);
					//$dateTmp->setTimezone($tz);

					$date = $match['date'];

					// not yet scheduled
					if ($match['time'] == '-:-') {
						$matchTime = $match['time'];
						$unterminiertInfo += 1;
					} else {
						$matchTime = $dateTmp->format('H:i');
					}

					$matchDayRound = [];

					$matchDayRound[] = $match['competition'];

					if (empty($match['matchday'])) {
						if (!empty($match['round'])) {
							$matchDayRound[] = $match['round'];
						}
					} else {
						$matchDayRound[] = $match['matchday'] . '. Spieltag';
					}

					if ($match['time'] == '-:-') {
						$matchDate = $dateTmp->format('d.m.Y') . ' *';
					} else {
						$matchDate = $dateTmp->format('d.m.Y');
					}

					$output .= '<div class="row schedule">';
					$output .= '<div>' . implode(', ', $matchDayRound) . '</div>';
					$output .= '<div>' . $matchDate . '</div>';
					$output .= '</div>';
				}

				$style = 'background:transparent';

				$resultBox = '<a href="' . $match['links']['compare'] . '" target="_blank" title="' . $match['homeTeamName'] . ' vs ' . $match['awayTeamName'] . ' - Formvergleich auf fussballdaten.de"><span class="time">' . $matchTime . '</span></a>';

				if ($match['status'] != 'TIMED' && $match['result']['goalsHomeTeam'] !== '') {
					$homeScore = $match['result']['goalsHomeTeam'];
					$awayScore = $match['result']['goalsAwayTeam'];

					$resultBox = '<a href="' . $match['links']['match'] . '" target="_blank" title="' . $match['homeTeamName'] . ' vs ' . $match['awayTeamName'] . ' - Aufstellungen & Spieldaten auf fussballdaten.de"><span class="result">' . $homeScore . ':' . $awayScore . '</span></a>';
				}

				if ($match['status'] == 'FINISHED') {
					$htmlClass += ' finished';
				}

				$output .= '<div class="row data ' . $rowcls . '" style="' . $style . '">';

				$homeLogo = '<img src="' . $match['homeTeamLogo'] . '" width="20" height="20" alt="Logo ' . $match['homeTeamName'] . '" />';
				if ($rowcls != 'full') {
					$homeLogo = '';
				}

				$output .= '<div><div class="flex">' . self::teamNameClass($rowcls, $match['homeTeamName'], $match['homeTeamShort']) . $homeLogo . '</div></div>';
				$output .= '<div class="' . $htmlClass . '">' . $resultBox . '</div>';

				$awayLogo = '<img src="' . $match['awayTeamLogo'] . '" width="20" height="20" alt="Logo ' . $match['homeTeamName'] . '" />';
				if ($rowcls != 'full') {
					$awayLogo = '';
				}

				$output .= '<div><div class="flex">' . $awayLogo . self::teamNameClass($rowcls, $match['awayTeamName'], $match['awayTeamShort']) . '</div></div>';
				$output .= '</div>';
			}
		}

		if ($unterminiertInfo > 0) {
			$output .= '<div style="font-size:12px;color:666;">*= unterminiert</div>';
		}

		$output .= '</div>';
		$output .= '</div>';

		if (isset($cache['links'])) {
			$output .= $this->getHtmlLinks($cache['links'], $atts, $css);
		}

		return $output;
	}

	public function getHtmlTable($atts, $css) {

		$cache = Fussballdaten::get_instance()->getCache()['table'][$atts['competition']];

		//echo "<pre>" . print_r($cache['standing'][0]) . "</pre>";

		if (!isset($atts['headline'])) {
			$output = '<div class="widget-title block-title"><h3>Tabelle</h3></div>';
		} elseif ($atts['headline'] == 'false' || empty($atts['headline'])) {
			$output = '';
		} else {
			$output = '<div class="widget-title block-title"><h3>' . $atts['headline'] . '</h3></div>';
		}

		$output .= '<div class="fdwp-wrapper">';
		$output .= '<div class="fdwp-table">';
		// header row
		if (isset($atts['display']) && $atts['display'] == 'full') {
			$rowcls = 'full';
		} else {
			$rowcls = 'mini';
		}

		$output .= '<div class="row date' . ((isset($atts['lazy-load']) && $atts['lazy-load']) ? ' hide' : '') . ' ' . $rowcls . '" style="' . $css['tableHead'] . '">';
		$output .= '<div class="cell col-1">#</div>';
		$output .= '<div class="cell col-6">Name</div>';
		$output .= '<div class="cell col-1">Sp</div>';

		if (isset($atts['display']) && $atts['display'] == 'full') {
			$output .= '<div class="cell col-1">G</div>';
			$output .= '<div class="cell col-1">U</div>';
			$output .= '<div class="cell col-1">V</div>';
			$output .= '<div class="cell col-1">Tore</div>';
		}

		$output .= '<div class="cell col-1">Diff</div>';
		$output .= '<div class="cell col-1">Pkt</div>';
		$output .= '</div>';

		if (isset($cache['standing'])) {

			foreach ($cache['standing'] as $k => $standing) {

				$style = ($k % 2 == 0) ? $css['tableRow'] : $css['tableRowStripe'];

				$logo = '<img src="' . $standing['teamLogo'] . '" width="20" height="20" alt="Logo ' . $standing['teamName'] . '" />';
				if ($rowcls == 'mini') {
					$logo = '';
				}

				$output .= '<div class="row ' . $rowcls . '" style="' . $style . '">';
				$output .= '<div class="cell">' . $standing['position'] . '</div>';
				$output .= '<div class="cell"><div class="flex">' . $logo . self::teamNameClass($rowcls, $standing['teamName'], $standing['teamNameShort']) . '</div></div>';
				$output .= '<div class="cell">' . $standing['playedGames'] . '</div>';

				if (isset($atts['display']) && $atts['display'] == 'full') {
					$output .= '<div class="cell center">' . $standing['wins'] . '</div>';
					$output .= '<div class="cell center">' . $standing['draws'] . '</div>';
					$output .= '<div class="cell center">' . $standing['losses'] . '</div>';
					$output .= '<div class="cell center">' . $standing['goals'] . ':' . $standing['goalsAgainst'] . '</div>';
				}

				$output .= '<div class="cell center">' . $standing['goalDifference'] . '</div>';
				$output .= '<div class="cell center">' . $standing['points'] . '</div>';
				$output .= '</div>';
			}
		}

		$output .= '</div>';
		$output .= '</div>';

		if (isset($cache['links'])) {
			$output .= $this->getHtmlLinks($cache['links'], $atts, $css);
		}

		return $output;
	}

	public function getHtmlLinks($links, $atts, $css = []) {

		#echo "<pre>";
		#print_r($cache);
		#echo '</pre>';

		if (isset($links)) {

			if (isset($atts['links']) && $atts['links'] == 'inline') {
				$cls = 'inline';
			} else {
				$cls = '';
			}

			$output = '<div class="fdwp-wrapper links">';

			$output .= '<div class="external">';
			$output .= '<div class="fdwp-title"><h3 class="fdwp-heading">Externe Links</h3></div>';
			$output .= '<ul class="fdwp-list ' . $cls . '">';
			foreach ($links as $link) {
				$output .= '<li class="cat-item">';
				$output .= '<svg xmlns="http://www.w3.org/2000/svg" x="0px" y="0px" width="12" height="12" viewBox="0 0 26 26">
															<path d="M 12.3125 0 C 10.425781 0.00390625 10.566406 0.507813 11.5625 1.5 L 14.78125 4.71875 L 9.25 10.25 C 8.105469 11.394531 8.105469 13.230469 9.25 14.375 L 11.6875 16.78125 C 12.832031 17.921875 14.667969 17.925781 15.8125 16.78125 L 21.34375 11.28125 L 24.5 14.4375 C 25.601563 15.539063 26 15.574219 26 13.6875 L 26 3.40625 C 26 -0.03125 26.035156 0 22.59375 0 Z M 4.875 5 C 2.183594 5 0 7.183594 0 9.875 L 0 21.125 C 0 23.816406 2.183594 26 4.875 26 L 16.125 26 C 18.816406 26 21 23.816406 21 21.125 L 21 14.75 L 18 17.75 L 18 21.125 C 18 22.160156 17.160156 23 16.125 23 L 4.875 23 C 3.839844 23 3 22.160156 3 21.125 L 3 9.875 C 3 8.839844 3.839844 8 4.875 8 L 8.3125 8 L 11.3125 5 Z"></path>
															</svg>';
				//$output .= ' <a href="' . $link['link'] . '" target="_blank" title="' . $link['title'] . '" style="'..'">' . $link['name'] . '</a>';
				$output .= ' <a href="' . $link['link'] . '" target="_blank" title="' . $link['title'] . '">' . $link['name'] . '</a>';
				$output .= '</li>';
			}
			$output .= '</ul>';
			$output .= '</div>';
			$output .= '</div>';
		} else {
			$output = '';
		}
		return $output;
	}

}
